function res = gmatrix_ss(w,theta0)
%Requires Sims (2002) gensys package to work.
%Inputs:
    %theta0 - vector of parameter values at theta0. Note that this code assumes
    %that the steady state inflation is already squared (for numerical stability).
    %Computes G-bar for the base case. 
    %w - vector of frequencies at which the G matrix should be approximated (e.g., on p. 17, we have
    % -4999pi/5000 to 4999pi/5000, so that in our case
    %w=2*pi*(-4999:1:4999)'/10000).

%% parameter setup 

tau = theta0(1);
beta=theta0(2);
nu = theta0(3); 
phi= theta0(4);
pistar=theta0(5);
psi1 = theta0(6);
psi2 = theta0(7);
rhor = theta0(8);
rhog = theta0(9);
rhoz = theta0(10);
sig2r = theta0(11);
sig2g = theta0(12);
sig2z = theta0(13);
kap=(tau*(1-nu))/(nu*phi*pistar); %compute kappa. NOTE: not pistar^2!
nt=length(theta0);

%% DSGE system solution using gensys
Gam0=[ 1 0 0 0 0 0 0 0;
0 1 0 0 0 0 0 0;
0 (1-rhor)*psi2 1 -(1-rhor)*psi2 -(1-rhor)*psi1 0 0 0;
0 -kap 0 kap -1 0 beta 0 ;
rhoz/tau (1-rhog) -1/tau -1 0 0 1/tau 1 ;
0 0 0 0 0 0 0 0;
0 0 0 0 1 0 0 0
0 0 0 1 0 0 0 0;
];
Gam1=[ rhoz 0 0 0 0 0 0 0 ;
0 rhog 0 0 0 0 0 0 ;
0 0 rhor 0 0 0 0 0;
0 0 0 0 0 0 0 0 ;
0 0 0 0 0 0 0 0 ;
0 -1 0 1 0 -1 0 0 ;
0 0 0 0 0 0 1 0;
0 0 0 0 0 0 0 1;
];
Gam2=zeros(8,3); Gam2(1:3,1:3)=eye(3);
Gam3=zeros(8,2); Gam3(7:8,7:8)=eye(2);
cons=zeros(8,1);
[G1,C,impact,fmat,fwt,ywt,gev,eu]=gensys(Gam0,Gam1,cons,Gam2,Gam3); %compute DSGE solution

%% Compute spectrum using theta0 at all frequencies
A=[100,0,0,0,0,0,0,0;
    0,0,0,0,400,0,0,0;
    0,0,400,0,0,0,0,0]; %construct the A matrix as on p. 20, except for the lag operator term,
                        %which is  added in a loop later
SIG=[sig2z,0,0;0,sig2g,0;0,0,sig2r]; %covariance matrix
spec=zeros(3*length(w),3);
i=1;
while i<=length(w); %loop computes spectra and stacks them in 1 big matrix
exe=exp(-1i*w(i));
A(1,4)=100 - 100*exe;
H=A*((eye(8) - G1*exe)\impact);
spec(((i-1)*3+1):(i*3),:)=(1/(2*pi))*H*SIG*H'; %compute spectrum using representation (4) in the paper
i=i+1;
end;

%% Numerical derivatives (simple two-point method)
dspec=zeros(3*length(w),3*nt);
h=theta0*1e-7; %numerical differentiation step
j=1;
while j<=nt; %the loop computes spectra at a perturbed theta vector
    theta=theta0;
    theta(j)=theta0(j)+h(j);
    tau = theta(1);
beta=theta(2);
nu = theta(3); 
phi= theta(4);
pistar=theta(5);
psi1 = theta(6);
psi2 = theta(7);
rhor = theta(8);
rhog = theta(9);
rhoz = theta(10);
sig2r = theta(11);
sig2g = theta(12);
sig2z = theta(13);
kap=(tau*(1-nu))/(nu*phi*pistar);
Gam0=[ 1 0 0 0 0 0 0 0;
0 1 0 0 0 0 0 0;
0 (1-rhor)*psi2 1 -(1-rhor)*psi2 -(1-rhor)*psi1 0 0 0;
0 -kap 0 kap -1 0 beta 0 ;
rhoz/tau (1-rhog) -1/tau -1 0 0 1/tau 1 ;
0 0 0 0 0 0 0 0;
0 0 0 0 1 0 0 0
0 0 0 1 0 0 0 0;
];
Gam1=[ rhoz 0 0 0 0 0 0 0 ;
0 rhog 0 0 0 0 0 0 ;
0 0 rhor 0 0 0 0 0;
0 0 0 0 0 0 0 0 ;
0 0 0 0 0 0 0 0 ;
0 -1 0 1 0 -1 0 0 ;
0 0 0 0 0 0 1 0;
0 0 0 0 0 0 0 1;
];
Gam2=zeros(8,3); Gam2(1:3,1:3)=eye(3);
Gam3=zeros(8,2); Gam3(7:8,7:8)=eye(2);
cons=zeros(8,1);
    [G1,C,impact,fmat,fwt,ywt,gev,eu]=gensys(Gam0,Gam1,cons,Gam2,Gam3);
A=[100,0,0,0,0,0,0,0;
    0,0,0,0,400,0,0,0;
    0,0,400,0,0,0,0,0]; 
SIG=[sig2z,0,0;0,sig2g,0;0,0,sig2r];
    i=1;
    while i<=length(w);
        exe=exp(-1i*w(i));
A(1,4)=100 - 100*exe;
H=A*((eye(8) - G1*exe)\impact);
dspec((i-1)*3+1:i*3,(j-1)*3+1:j*3)=(1/(2*pi))*H*SIG*H'; 
i=i+1;
    end
    j=j+1;
end
G=zeros(nt,nt);
j=1;
while j<=nt; %computing G using representation in Remark 3 and approximating the integral numerically
    k=1;
    while k<=nt;
       aj=(dspec(:,((j-1)*3+1):(j*3))-spec)/h(j);
       ak=(dspec(:,((k-1)*3+1):(k*3))-spec)/h(k); 
       f=1;
   while f<=(rows(spec)/3);
G(j,k)=G(j,k)+trace(aj(((f-1)*3+1):(f*3),:)*ak(((f-1)*3+1):(f*3),:));
f=f+1;
   end
   k=k+1;
    end
    j=j+1;
end
G2=2*pi*G./length(w); 
res=G2+mufun(1:14); %add on the miu term